@@ -1,7 +1,7 @@
 'use strict';
-var reprs = require('./reprs.js');
+var bignum = require('bignum-browserify');
 
-var bignum = require('bignum');
+var sslBignum = require('bignum');
 
 var core = require('./core.js');
 
@@ -9,19 +9,6 @@ var hll = require('./hll.js');
 
 var op = {};
 exports.op = op;
-
-op.box_i = function(i, type) {
-  var repr = type._STable.REPR;
-  var obj = repr.allocate(type._STable);
-  obj.$$setInt(i);
-  return obj;
-};
-
-op.unbox_i = function(obj) {
-  if (typeof obj == 'number') return obj;
-  return obj.$$getInt();
-};
-
 function intishBool(b) {
   return b ? 1 : 0;
 }
@@ -96,7 +83,13 @@ op.sub_I = function(a, b, type) {
 };
 
 op.div_I = function(a, b, type) {
-  return makeBI(type, getBI(a).div(getBI(b)));
+  let divident = getBI(a);
+  let divisor = getBI(b);
+  // workaround for .div rounding to zero not down
+  if (divisor.lt(0) != divident.lt(0) && divident.mod(divisor).ne(0)) {
+    return makeBI(type, divident.div(divisor).sub(1));
+  }
+  return makeBI(type, divident.div(divisor));
 };
 
 op.pow_I = function(a, b, numType, biType) {
@@ -119,7 +112,7 @@ op.pow_I = function(a, b, numType, biType) {
 
 op.mod_I = function(n, m, type) {
   /* TODO - think if this can be optimized. */
-  /* We are doing this in complicated because,
+  /* We are doing this in complicated way because,
      bignum returns the module with the sign equal to the dividend not the divisor. */
   var a = getBI(n);
   var b = getBI(m);
@@ -147,12 +140,16 @@ op.expmod_I = function(a, b, c, type) {
 };
 
 op.div_In = function(a, b) {
-  var digits = 1e+20;
-  return getBI(a).mul(bignum(digits)).div(getBI(b)).toNumber() / digits;
+  let digits = 1e+20;
+  let divisor = getBI(b);
+  if (divisor.eq(0)) {
+    return getBI(a).toNumber() / 0;
+  }
+  return getBI(a).mul(bignum(digits)).div(divisor).toNumber() / digits;
 };
 
 op.rand_I = function(max, type) {
-  return makeBI(type, getBI(max).rand());
+  return makeBI(type, bignum(sslBignum(getBI(max).toString()).rand().toString()));
 };
 
 op.isle_I = function(a, b) {
@@ -177,19 +174,27 @@ op.cmp_I = function(a, b) {
 };
 
 op.isprime_I = function(n) {
-  return intishBool(getBI(n).probPrime(50));
+  return intishBool(sslBignum(getBI(n).toString()).probPrime(50));
 };
 
 op.bitshiftl_I = function(a, b, type) {
-  return makeBI(type, getBI(a).shiftLeft(b));
+  let n = getBI(a);
+  if (b < 0 && n.lt(0)) {
+    let shifted = n.shiftLeft(b);
+    return makeBI(type, shifted.shiftRight(b).eq(n) ? shifted : shifted.sub(1));
+  } else {
+    return makeBI(type, n.shiftLeft(b));
+  }
 };
 
 op.bitshiftr_I = function(a, b, type) {
-  return makeBI(type, getBI(a).shiftRight(b));
-};
-
-op.bitshiftr_I = function(a, b, type) {
-  return makeBI(type, getBI(a).shiftRight(b));
+  let n = getBI(a);
+  if (b >= 0 && n.lt(0)) {
+    let shifted = n.shiftRight(b);
+    return makeBI(type, shifted.shiftLeft(b).eq(n) ? shifted : shifted.sub(1));
+  } else {
+    return makeBI(type, n.shiftRight(b));
+  }
 };
 
 op.bitand_I = function(a, b, type) {
@@ -229,9 +234,9 @@ op.tonum_I = function(n) {
 op.fromnum_I = function(num, type) {
   // node-bignum bug workaround, when a negative number is too big it gets turned into 0
   if (num < 0) {
-    return makeBI(type, bignum(-num).neg());
+    return makeBI(type, bignum(Math.floor(-num)).neg());
   } else {
-    return makeBI(type, bignum(num));
+    return makeBI(type, bignum(Math.floor(num)));
   }
 };
 
@@ -262,7 +267,6 @@ op.radix_I = function(hllName, radix, str, zpos, flags, type) {
 
       result = result.add(base.mul(digit));
       base = base.mul(radix);
-
     }
 
     if (n[0] == '-') result = result.neg();
