@@ -14,6 +14,7 @@ const NUMERIC = xregexp('^\\p{Nd}');
 const HEXADECIMAL = xregexp('^[0-9A-Fa-f]');
 const WHITESPACE = xregexp('^\\p{White_Space}');
 const BLANK = xregexp('^[\t\\p{Zs}]');
+const NEWLINE = xregexp('[\n\r\u0085\u2029\f\u000b\u2028]');
 const PUNCTUATION = xregexp('^\\pP');
 const ALPHANUMERIC = xregexp('^[\\pL\\p{Nd}]');
 const WORD = xregexp('^[\\pL_\\p{Nd}]');
@@ -21,33 +22,39 @@ const WORD = xregexp('^[\\pL_\\p{Nd}]');
 function iscclass(cclass, target, offset) {
   if (offset < 0 || offset >= target.length) return 0;
   switch (cclass) {
-    //ANY
+    // ANY
     case 65535: return 1;
-    //UPPERCASE
+    // UPPERCASE
     case 1: return boolish(UPPERCASE.test(target[offset]));
-    //LOWERCASE
+    // LOWERCASE
     case 2: return boolish(LOWERCASE.test(target[offset]));
-    //ALPHABETIC
+    // ALPHABETIC
     case 4: return boolish(ALPHABETIC.test(target[offset]));
-    //NUMERIC
+    // NUMERIC
     case 8: return boolish(NUMERIC.test(target[offset]));
-    //HEXADECIMAL
+    // HEXADECIMAL
     case 16: return boolish(HEXADECIMAL.test(target[offset]));
-    //WHITESPACE
+    // WHITESPACE
     case 32: return boolish(WHITESPACE.test(target[offset]));
-    //BLANK
+    // BLANK
     case 256: return boolish(BLANK.test(target[offset]));
-    //CONTROL
-    /* TODO - all character*/
-    case 512: return boolish('\n\t\r'.indexOf(target[offset]) != -1); //HACK - not all such chars
-    //PUNCTUATION
-    case 1024: return boolish(PUNCTUATION.test(target[offset])); //HACK
-    //ALPHANUMERIC
+    // PRINTING
+    case 64: {
+      let cp = target.codePointAt(offset);
+      return boolish(!((cp >= 0 && cp < 32) || (cp >= 127 && cp < 160)));
+    }
+    // CONTROL
+    case 512: {
+      let cp = target.codePointAt(offset);
+      return boolish((cp >= 0 && cp < 32) || (cp >= 127 && cp < 160));
+    }
+    // PUNCTUATION
+    case 1024: return boolish(PUNCTUATION.test(target[offset])); // HACK
+    // ALPHANUMERIC
     case 2048: return boolish(ALPHANUMERIC.test(target[offset]));
-    //NEWLINE
-    /* TODO - all characters*/
-    case 4096: return boolish(target[offset] == '\n' || target[offset] == '\r' || target[offset] == '\u0085'); //HACK
-    //WORD
+    // NEWLINE
+    case 4096: return boolish(NEWLINE.test(target[offset]));
+    // WORD
     case 8192: return boolish(WORD.test(target[offset]));
     default: throw 'cclass ' + cclass + ' not yet implemented';
   }
