@@ -1,6 +1,8 @@
 var op = {};
 exports.op = op;
 
+var os = require('os');
+
 var Hash = require('./hash.js');
 var CodeRef = require('./code-ref.js');
 
@@ -16,10 +18,6 @@ var hll = require('./hll.js');
 
 var NQPObject = require('./nqp-object.js');
 
-var bootstrap = require('./bootstrap.js');
-
-var nqp = require('nqp-runtime');
-
 var constants = require('./constants.js');
 
 var containerSpecs = require('./container-specs.js');
@@ -32,6 +30,10 @@ var exceptionsStack = require('./exceptions-stack.js');
 
 var repossession = require('./repossession.js');
 
+var sixmodel = require('./sixmodel.js');
+
+var Capture = require('./capture.js');
+
 exports.CodeRef = CodeRef;
 
 op.isinvokable = function(obj) {
@@ -120,7 +122,7 @@ exports.named = function(parts) {
 };
 
 exports.op.ishash = function(obj) {
-  return obj instanceof Hash ? 1 : 0;
+  return obj instanceof Hash || (obj._STable && obj._STable.REPR instanceof reprs.VMHash) ? 1 : 0;
 };
 
 op.existspos = function(array, idx) {
@@ -157,16 +159,6 @@ op.setboolspec = function(obj, mode, method) {
   return obj;
 };
 
-class Capture extends NQPObject {
-  constructor(named, pos, skip) {
-    super();
-    this.pos = pos;
-    this.named = named;
-  }
-  $$clone() {
-    return this;
-  }
-};
 
 op.savecapture = function(args) {
   return new Capture(args[1], Array.prototype.slice.call(args, 2));
@@ -180,6 +172,18 @@ op.captureposarg = function(capture, i) {
   return capture.pos[i];
 };
 
+op.captureposarg_i = function(capture, i) {
+  return op.unbox_i(capture.pos[i]);
+};
+
+op.captureposarg_s = function(capture, i) {
+  return op.unbox_s(capture.pos[i]);
+};
+
+op.captureposarg_n = function(capture, i) {
+  return op.unbox_n(capture.pos[i]);
+};
+
 op.capturehasnameds = function(capture) {
   return (!capture.named || Object.keys(capture.named).length == 0) ? 0 : 1;
 };
@@ -204,57 +208,15 @@ op.getcodeobj = function(codeRef) {
   return codeRef.codeObj;
 };
 
-
-op.istype = function(ctx, obj, type) {
-  /* Null always type checks false. */
-  if (obj === Null) {
-    return 0;
-  }
-
-  // HACK
-  if (typeof obj === 'number' || typeof obj === 'string' || obj instanceof Hash || obj instanceof NQPInt) {
-    return 0;
-  }
-
-  var cache = obj._STable.typeCheckCache;
-  if (cache) {
-    for (var i = 0; i < cache.length; i++) {
-      if (cache[i] === type) {
-        return 1;
-      }
-    }
-  } else {
-    /* If we get here, need to call .^type_check on the value we're
-     * checking. */
-
-    var HOW = obj._STable.HOW;
-    /* This "hack" is stolen from the JVM */
-    if (!HOW.type_check) {
-      return 0;
-    }
-
-    if (HOW.type_check(ctx, null, HOW, obj, type).$$toBool(ctx)) {
-      return 1;
-    }
-  }
-
-  if (type._STable.typeCheckNeedsAccepts) {
-    return type._STable.HOW.accepts_type(ctx, null, type._STable.HOW, type, obj).$$toBool(ctx);
-  }
-
-  return 0;
-};
-
 op.settypecache = function(obj, cache) {
   obj._STable.typeCheckCache = cache.array;
-  if (obj._STable._SC !== null) obj._STable.scwb();
+  if (obj._STable._SC !== undefined) obj._STable.scwb();
   return obj;
 };
 
 op.settypecheckmode = function(obj, mode) {
-  if (mode & 2) {
-    obj._STable.typeCheckNeedsAccepts = true;
-  }
+  const TYPE_CHECK_CACHE_FLAG_MASK = 3;
+  obj._STable.modeFlags = mode | (obj.STable.modeFlags & TYPE_CHECK_CACHE_FLAG_MASK);
   return obj;
 };
 
@@ -262,8 +224,8 @@ op.setmethcache = function(obj, cache) {
   if (!cache instanceof Hash) {
     console.log('we expect a hash here');
   }
-  obj._STable.setMethodCache(cache.$$toObject());
-  if (obj._STable._SC !== null) obj._STable.scwb();
+  obj._STable.setMethodCache(cache.content);
+  if (obj._STable._SC !== undefined) obj._STable.scwb();
   return obj;
 };
 
@@ -273,7 +235,7 @@ op.setmethcacheauth = function(obj, isAuth) {
   } else {
     obj._STable.modeFlags &= ~constants.METHOD_CACHE_AUTHORITATIVE;
   }
-  if (obj._STable._SC !== null) obj._STable.scwb();
+  if (obj._STable._SC !== undefined) obj._STable.scwb();
   return obj;
 };
 
@@ -303,35 +265,10 @@ op.newtype = function(how, repr) {
   return REPR.typeObjectFor(how);
 };
 
-function find_method(ctx, obj, name) {
-  if (obj._STable.methodCache) {
-    var hasMethod = obj._STable.methodCache.hasOwnProperty(name);
-    if (hasMethod) {
-      return obj._STable.methodCache[name];
-    }
-    if (obj._STable.modeFlags & constants.METHOD_CACHE_AUTHORITATIVE) {
-      return Null;
-    }
-  }
-
-  if (obj._STable.HOW.find_method) {
-    return obj._STable.HOW.find_method(ctx, null, obj._STable.HOW, obj, name);
-  } else {
-    return Null;
-  }
-}
-
-op.can = function(ctx, obj, name) {
-  /* HACK those things should have an STable rather then be special cased */
-  if (typeof obj !== 'object' || obj instanceof NQPInt || obj instanceof CodeRef || obj instanceof Hash) return 0;
-
-  return find_method(ctx, obj, name) === Null ? 0 : 1;
-};
-
 op.findmethod = function(ctx, obj, name) {
-  var method = find_method(ctx, obj, name);
+  var method = sixmodel.findMethod(ctx, obj, name);
   if (method === Null) {
-    throw new NQPException("Cannot find method '" + name + "' on object of type " + obj._STable.debugName);
+    throw new NQPException(`Cannot find method '${name}' on object of type '${obj._STable.debugName}'`);
   }
   return method;
 };
@@ -347,7 +284,7 @@ op.setcodename = function(code, name) {
 
 op.rebless = function(obj, newType) {
   obj._STable.REPR.changeType(obj, newType);
-  if (obj._SC !== null) obj.$$scwb();
+  if (obj._SC !== undefined) obj.$$scwb();
   return obj;
 };
 
@@ -355,16 +292,6 @@ op.composetype = function(obj, reprinfo) {
   obj._STable.REPR.compose(obj._STable, reprinfo);
 };
 
-op.clone = function(obj) {
-  if (obj.$$clone) {
-    return obj.$$clone();
-  } else {
-    // STUB
-    console.log('NYI cloning', obj);
-    return obj;
-  }
-};
-
 var whereCounter = 0;
 op.where = function(obj) {
   if (obj._STable || obj instanceof CodeRef) { // HACK
@@ -397,7 +324,7 @@ op.setcontspec = function(type, specType, conf) {
     type._STable.containerSpec = new containerSpecs[specType](type._STable);
     type._STable.containerSpec.configure(conf);
   } else {
-    console.warn('NYI cont spec: ' + specType);
+    throw 'NYI cont spec: ' + specType;
   }
 };
 
@@ -423,7 +350,7 @@ op.iscont_s = function(cont) {
 };
 
 op.isrwcont = function(cont) {
-  return cont.$$decont ? 1 : 0;
+  return cont.$$isrwcont ? cont.$$isrwcont() : 0;
 };
 
 op.box_n = function(n, type) {
@@ -434,6 +361,7 @@ op.box_n = function(n, type) {
 };
 
 op.unbox_n = function(obj) {
+  if (typeof obj == 'number') return obj;
   return obj.$$getNum();
 };
 
@@ -449,8 +377,17 @@ op.unbox_s = function(obj) {
   return obj.$$getStr();
 };
 
-op.elems = function(obj) {
-  return obj.$$elems();
+
+op.box_i = function(i, type) {
+  var repr = type._STable.REPR;
+  var obj = repr.allocate(type._STable);
+  obj.$$setInt(i);
+  return obj;
+};
+
+op.unbox_i = function(obj) {
+  if (typeof obj == 'number') return obj;
+  return obj.$$getInt();
 };
 
 op.setelems = function(obj, elems) {
@@ -541,12 +478,11 @@ function toJS(obj) {
   }
 }
 
+/* For use inside the eval */
+const nqp = require('nqp-runtime');
+
 class JavaScriptCompiler extends NQPObject {
   eval(ctx, _NAMED, self, code) {
-    if (/PRINTME/.test(code)) {
-      console.log('evaling [', code, ']');
-    }
-    //console.log("evaling [", code, "]");
     return fromJS(eval(code));
   }
 };
@@ -578,6 +514,7 @@ op.getcomp = function(language) {
 op.backendconfig = function() {
   var config = new Hash();
   config.content.set('intvalsize', 4);
+  config.content.set('osname', os.platform());
   return config;
 };
 
@@ -590,19 +527,19 @@ op.getpid = function() {
 };
 
 op.getmessage = function(exception) {
-  return exception.message;
+  return exception.$$message;
 };
 
 op.setmessage = function(exception, message) {
-  return (exception.message = message);
+  return (exception.$$message = message);
 };
 
 op.getpayload = function(exception) {
-  return Object.prototype.hasOwnProperty.call(exception, 'payload') ? exception.payload : Null;
+  return Object.prototype.hasOwnProperty.call(exception, '$$payload') ? exception.$$payload : Null;
 };
 
 op.setpayload = function(exception, payload) {
-  return (exception.payload = payload);
+  return (exception.$$payload = payload);
 };
 
 op.isnum = function(value) {
@@ -614,11 +551,17 @@ op.isint = function(value) {
 };
 
 function renameEncoding(encoding) {
-  return {utf16: 'utf16le', 'iso-8859-1': 'binary'}[encoding] || encoding;
+  return {'utf16': 'utf16le', 'iso-8859-1': 'binary'}[encoding] || encoding;
 }
-
 exports.renameEncoding = renameEncoding;
 
+let encodings = ['ascii', 'utf8', 'utf16le', 'ucs2', 'base64', 'latin1', 'binary', 'hex'];
+function isKnownEncoding(encoding) {
+  return encodings.indexOf(encoding) == -1 ? false : true;
+}
+exports.isKnownEncoding = isKnownEncoding;
+
+
 function byteSize(buf) {
   var bits = buf._STable.REPR.type._STable.REPR.bits;
 
@@ -629,44 +572,71 @@ function byteSize(buf) {
   return bits / 8;
 }
 
+exports.byteSize = byteSize;
+
 op.encode = function(str, encoding_, buf) {
+  if (buf.array.length) {
+    throw new NQPException('encode requires an empty array');
+  }
+
   var encoding = renameEncoding(encoding_);
 
   var elementSize = byteSize(buf);
 
-  var ret = [];
+  var isUnsigned = buf._STable.REPR.type._STable.REPR.isUnsigned;
 
   var buffer = new Buffer(str, encoding);
 
   var offset = 0;
   for (var i = 0; i < buffer.length / elementSize; i++) {
-    buf.array[i] = buffer.readIntLE(offset, elementSize);
+    buf.array[i] = isUnsigned ? buffer.readUIntLE(offset, elementSize) : buffer.readIntLE(offset, elementSize);
     offset += elementSize;
   }
 
   return buf;
 };
 
-op.decode = function(buf, encoding_) {
-  var encoding = renameEncoding(encoding_);
-  var elementSize = byteSize(buf);
+function toRawBuffer(buf) {
+  let elementSize = byteSize(buf);
+  let isUnsigned = buf._STable.REPR.type._STable.REPR.isUnsigned;
+  let array = buf.array;
 
-  buf = buf.array;
+  let buffer = new Buffer(array.length * elementSize);
 
-  var buffer = new Buffer(buf.length * elementSize);
-
-  var offset = 0;
-  for (var i = 0; i < buf.length; i++) {
-    buffer.writeIntLE(buf[i], offset, elementSize);
+  let offset = 0;
+  for (let i = 0; i < array.length; i++) {
+    if (isUnsigned) {
+      buffer.writeUIntLE(array[i], offset, elementSize);
+    } else {
+      buffer.writeIntLE(array[i], offset, elementSize);
+    }
     offset += elementSize;
   }
 
-  return buffer.toString(encoding);
+  return buffer;
+}
+
+exports.toRawBuffer = toRawBuffer;
+
+op.decode = function(buf, encoding) {
+  return toRawBuffer(buf).toString(renameEncoding(encoding));
 };
 
 op.objprimspec = function(obj) {
   if (obj === Null) return 0;
-  return (obj._STable && obj._STable.REPR.boxedPrimitive ? obj._STable.REPR.boxedPrimitive : 0);
+  if (typeof obj === 'object') {
+    if (obj instanceof NQPInt) {
+      return 1;
+    } else {
+      return (obj._STable && obj._STable.REPR.boxedPrimitive ? obj._STable.REPR.boxedPrimitive : 0);
+    }
+  } else if (typeof obj == 'number') {
+    return 2;
+  } else if (typeof obj == 'string') {
+    return 3;
+  } else {
+    throw new NQPException(`objprimspec can't handle things of type: ${typeof obj}`);
+  }
 };
 
 /* Parametricity operations. */
@@ -705,7 +675,7 @@ op.parameterizetype = function(ctx, type, params) {
       var match = true;
       for (var j = 0; j < unpackedParams.length; j++) {
         /* XXX More cases to consider here. - copied over from the jvm backend, need to consider what they are*/
-        if (unpackedParams[j] != lookup[i].params[j]) {
+        if (unpackedParams[j] !== lookup[i].params[j]) {
           match = false;
           break;
         }
@@ -757,6 +727,7 @@ op.asec_n = function(x) {
 };
 
 op.sech_n = function(x) {
+  if (x == Infinity || x == -Infinity) return 0;
   return (2 * Math.cosh(x)) / (Math.cosh(2 * x) + 1);
 };
 
@@ -781,53 +752,55 @@ op.typeparameterat = function(ctx, type, idx) {
 
 op.typeparameterized = function(type) {
   var st = type._STable;
-  var nqp = require('nqp-runtime');
   return st.parametricType ? st.parametricType : Null;
 };
 
-function startTrampoline(thunk_) {
-  var thunk = thunk_;
-  while (thunk) {
-    thunk = thunk();
+let fibers = require('fibers');
+
+function runTagged(tag, fiber, val) {
+  var control = fiber.run(val);
+  while (1) {
+    if (control.tag == tag || control.tag === Null) {
+      return control.value;
+    } else {
+      fibers.yield(control);
+    }
   }
 }
 
-var resetValue;
-var invokeValue;
-op.continuationreset = function(ctx, tag, continuation) {
-  startTrampoline(function() {
-    return continuation.$$callCPS(ctx, {}, function(value) {
-      resetValue = value;
-      invokeValue = value;
-      return Null;
-    });
-  });
-  return resetValue;
+class Cont {
+  constructor(tag, fiber) {
+    this.tag = tag;
+    this.fiber = fiber;
+  }
+
+  $$decont(ctx) {
+    return this;
+  }
+
+  $$toBool(ctx) {
+    return 1;
+  }
 };
 
-op.continuationresetCPS = function(ctx, tag, continuation, continuation) {
-  console.log('continuation reset in CPS mode');
+op.continuationreset = function(ctx, tag, block) {
+  if (block instanceof Cont) {
+    return runTagged(tag, block.fiber, Null);
+  } else {
+    return runTagged(tag, fibers(function() {
+      return {value: block.$$call(ctx, null), tag: tag};
+    }), Null);
+  }
 };
 
-op.continuationcontrolCPS = function(ctx, protect, tag, run, cont) {
-  startTrampoline(run.$$callCPS(ctx, {}, function(value) {
-    resetValue = value;
-    return Null;
-  }, cont));
-  return Null;
+
+op.continuationcontrol = function(ctx, protect, tag, closure) {
+  return fibers.yield({value: closure.$$call(ctx, null, new Cont(tag, fibers.current)), tag: tag});
 };
 
 op.continuationinvoke = function(ctx, cont, inject) {
-  // TODO really place inject inside the cont
-  var value = inject.$$call(ctx, {});
-  startTrampoline(cont(value));
-  return invokeValue;
-};
-
-op.continuationinvokeCPS = function(ctx, invokedCont, inject, cont) {
-  // TODO really place inject inside the cont, use callCPS
-  var value = inject.$$call(ctx, {});
-  return invokedCont(value);
+  var val = inject.$$call(ctx, null);
+  return runTagged(cont.tag, cont.fiber, val);
 };
 
 var generator = Math;
@@ -949,13 +922,19 @@ op.setdimensions = function(array, dimensions) {
   };
 });
 
-var BOOTException = bootstrap.bootType('BOOTException', 'VMException');
 /* TODO HLL support */
 op.newexception = function() {
-  var exType = BOOTException;
+  var exType = BOOT.Exception;
   return exType._STable.REPR.allocate(exType._STable);
 };
 
+op.throwextype = function(ctx, category) {
+  var exType = BOOT.Exception;
+  let ex = exType._STable.REPR.allocate(exType._STable);
+  ex.$$category = category;
+  ctx.throw(ex);
+};
+
 function EvalResult(mainline, codeRefs) {
   this.mainline = mainline;
   this.codeRefs = codeRefs;
@@ -983,17 +962,72 @@ op.backtracestrings = function(hllName, exception) {
   return hll.list(hllName, [exception.stack.replace(/^Error: .*\n/, '')]);
 };
 
+op.backtrace = function(hllName, exception) {
+  /* TODO - have real file and line */
+  if (exception.$$ctx) {
+    let ctx = exception.$$ctx;
+
+    let rows = [];
+
+    while (ctx) {
+      let row = new Hash();
+      let annotations = new Hash();
+      row.content.set('annotations', annotations);
+      if (ctx instanceof Ctx) {
+        row.content.set('sub', ctx.codeRef());
+      }
+
+      annotations.content.set('file', 'NYI');
+      annotations.content.set('line', 0);
+
+      rows.push(row);
+
+      ctx = ctx.$$outer;
+    }
+    return hll.list(hllName, rows);
+  } else {
+    return hll.list(hllName, []);
+  }
+};
+
 op.hintfor = function(classHandle, attrName) {
   if (!classHandle._STable.REPR.hintfor) return -1;
   return classHandle._STable.REPR.hintfor(classHandle, attrName);
 };
 
 op.ctxcaller = function(ctx) {
-  return ctx.$$caller;
+  return ctx.$$caller.$$skipHandlers();
+};
+
+op.ctxcallerskipthunks = function(ctx) {
+  let caller = ctx.$$skipHandlers().$$caller;
+  if (caller) caller = caller.$$skipHandlers();
+
+  // FIXME - ctxs that don't have a codeRef
+  while (caller && caller.codeRef().staticCode.isThunk) {
+    caller = caller.$$caller;
+    if (caller) caller = caller.$$skipHandlers();
+  }
+  return caller || Null;
+};
+
+op.ctxouterskipthunks = function(ctx) {
+  var outer = ctx.$$skipHandlers().$$outer;
+
+  // FIXME - ctxs that don't have a codeRef
+  while (outer && outer.codeRef().staticCode.isThunk) {
+    outer = outer.$$outer;
+    if (outer) outer = outer.$$skipHandlers();
+  }
+  return outer || Null;
 };
 
 op.captureposprimspec = function(capture, idx) {
-  return 0;
+  if (capture.pos[idx].typeObject_) {
+    return 0;
+  } else {
+    return op.objprimspec(capture.pos[idx]);
+  }
 };
 
 op.forceouterctx = function(code, ctx) {
@@ -1004,88 +1038,56 @@ op.forceouterctx = function(code, ctx) {
     throw 'forceouterctx second operand must be a Ctx';
   }
 
-  code.forcedOuterCtx = ctx;
+  code.outerCtx = ctx;
 
   return code;
 };
 
-op.getattrref_i = function(hllName, get, set) {
-  var refType = hll.hllConfigs[hllName].get('int_attr_ref');
-  if (refType === undefined) {
-    throw 'No int attribute reference type registered for current HLL';
-  }
-  var STable = refType._STable;
-  var ref = STable.REPR.allocate(STable);
-  ref.get = get;
-  ref.set = set;
-  return ref;
-};
-
-op.getattrref_n = function(hllName, get, set) {
-  var refType = hll.hllConfigs[hllName].get('num_attr_ref');
-  if (refType === undefined) {
-    throw 'No num attribute reference type registered for current HLL';
-  }
-  var STable = refType._STable;
-  var ref = STable.REPR.allocate(STable);
-  ref.get = get;
-  ref.set = set;
-  return ref;
-};
-
-op.getattrref_s = function(hllName, get, set) {
-  var refType = hll.hllConfigs[hllName].get('str_attr_ref');
-  if (refType === undefined) {
-    throw 'No str attribute reference type registered for current HLL';
-  }
-  var STable = refType._STable;
-  var ref = STable.REPR.allocate(STable);
-  ref.get = get;
-  ref.set = set;
-  return ref;
-};
-
-op.getlexref_i = function(hllName, get, set) {
-  var refType = hll.hllConfigs[hllName].get('int_lex_ref');
-  if (refType === undefined) {
-    throw 'No int lexical reference type registered for current HLL';
-  }
-  var STable = refType._STable;
-  var ref = STable.REPR.allocate(STable);
-  ref.get = get;
-  ref.set = set;
-  return ref;
-};
-
-op.getlexref_s = function(hllName, get, set) {
-  var refType = hll.hllConfigs[hllName].get('str_lex_ref');
-  if (refType === undefined) {
-    throw 'No str lexical reference type registered for current HLL';
-  }
-  var STable = refType._STable;
-  var ref = STable.REPR.allocate(STable);
-  ref.get = get;
-  ref.set = set;
-  return ref;
-};
-
-op.getlexref_n = function(hllName, get, set) {
-  var refType = hll.hllConfigs[hllName].get('num_lex_ref');
-  if (refType === undefined) {
-    throw 'No num lexical reference type registered for current HLL';
-  }
-  var STable = refType._STable;
-  var ref = STable.REPR.allocate(STable);
-  ref.get = get;
-  ref.set = set;
-  return ref;
-};
-
 // TODO: replace it with a fully correct implementation
 op.fc = function(string) {
   return string.toLowerCase();
 };
 
+function tcChar(c) {
+  if (c === 'ß') return 'Ss';
+  var unicharadata = require('unicharadata');
+  var titled = unicharadata.title(c);
+  return titled === '' ? c : titled;
+}
+
+op.tc = function(string) {
+  var ret = '';
+  for (let c of string) {
+    ret += tcChar(c);
+  }
+  return ret;
+};
+
+op.tclc = function(string) {
+  let isFirst = true;
+  let lower = '';
+  let first = '';
+  for (let c of string) {
+    if (isFirst) {
+      isFirst = false;
+      first = tcChar(c);
+    } else {
+      lower += c;
+    }
+  }
+  return first + lower.toLowerCase();
+};
+
+op.getstrfromname = function(name) {
+  let unicharadata = require('unicharadata');
+  return unicharadata.lookup(name);
+};
+
+op.codepointfromname = function(name) {
+  let unicharadata = require('unicharadata');
+  return unicharadata.lookup(name).codePointAt(0);
+};
+
 op.islist = function(list) {
   return (list._STable && list._STable.REPR instanceof reprs.VMArray) ? 1 : 0;
 };
@@ -1097,3 +1099,26 @@ op.split = function(hllName, separator, string) {
 op.exception = function() {
   return exceptionsStack[exceptionsStack.length - 1];
 };
+
+op.lastexpayload = function() {
+  return exceptionsStack[exceptionsStack.length - 1].$$payload;
+};
+
+op.setextype = function(exception, category) {
+  exception.$$category = category;
+  return exception;
+};
+
+op.getextype = function(exception) {
+  if (exception.$$category === undefined) {
+    return 0;
+  } else {
+    return exception.$$category;
+  }
+};
+
+op.pow_n = function(base, exponent) {
+  if (exponent === Infinity && base === 1) return 1;
+  if (exponent === -Infinity && base === 1) return 1;
+  return Math.pow(base, exponent);
+};
