@@ -19,6 +19,22 @@ class NQPInt extends NQPObject {
   $$numify() {
     return this.value;
   }
+
+  $$istype(ctx, type) {
+    return 0;
+  }
+
+  $$can(ctx, name) {
+    return 0;
+  }
+
+  $$clone() {
+    return this;
+  }
+
+  $$getInt() {
+    return this.value;
+  }
 };
 
 module.exports = NQPInt;
