@@ -4,15 +4,34 @@ var Hash = require('./hash.js');
 var NQPInt = require('./nqp-int.js');
 var NQPException = require('./nqp-exception.js');
 var Null = require('./null.js');
-var null_s = require('./null_s.js');
+var nullStr = require('./null_s.js');
 var Iter = require('./iter.js');
 var BOOT = require('./BOOT.js');
+var core = require('./core.js');
+var nqp = require('nqp-runtime');
 
-var bignum = require('bignum');
+var bignum = require('bignum-browserify');
 var ZERO = bignum(0);
 
 var constants = require('./constants.js');
 
+const EDGE_FATE = 0;
+const EDGE_EPSILON = 1;
+const EDGE_CODEPOINT = 2;
+const EDGE_CODEPOINT_NEG = 3;
+const EDGE_CHARCLASS = 4;
+const EDGE_CHARCLASS_NEG = 5;
+const EDGE_CHARLIST = 6;
+const EDGE_CHARLIST_NEG = 7;
+const EDGE_SUBRULE = 8;
+const EDGE_CODEPOINT_I = 9;
+const EDGE_CODEPOINT_I_NEG = 10;
+const EDGE_GENERIC_VAR = 11;
+const EDGE_CHARRANGE = 12;
+const EDGE_CHARRANGE_NEG = 13;
+const EDGE_CODEPOINT_LL = 14;
+const EDGE_CODEPOINT_I_LL = 15;
+
 var reprs = {};
 var reprById = [];
 
@@ -27,19 +46,31 @@ function basicTypeObjectFor(HOW) {
 }
 
 function basicAllocate(STable) {
-  return new STable.objConstructor();
+  return new STable.ObjConstructor();
 }
 
 function noopCompose(obj, reprInfo) {
 }
 
+function methodNotFoundError(ctx, obj, name) {
+  let handler = ctx ? ctx.$$getHLL().get('method_not_found_error') : undefined;
+  if (handler === undefined) {
+    throw new NQPException(`Cannot find method '${name}' on object of type ${obj._STable.debugName}`);
+  } else {
+    handler.$$call(ctx, null, obj, name);
+  }
+}
 
 function basicConstructor(STable) {
-  var objConstructor = function() {};
+  var ObjConstructor = function() {};
   var handler = {};
   handler.get = function(target, name) {
-    if (STable.modeFlags & constants.METHOD_CACHE_AUTHORITATIVE) {
-      return undefined;
+    if (STable.lazyMethodCache) {
+      STable.setMethodCache(STable.methodCache);
+      let method = STable.methodCache.get(name);
+      if (method !== undefined) {
+        return STable.ObjConstructor.prototype[name];
+      }
     }
 
     /* are we trying to access an internal property? */
@@ -47,11 +78,22 @@ function basicConstructor(STable) {
       return undefined;
     }
 
+    if (STable.modeFlags & constants.METHOD_CACHE_AUTHORITATIVE) {
+      return function(ctx, _NAMED, obj) {
+        methodNotFoundError(ctx, obj, name);
+      };
+    }
+
+
     return function() {
       let how = this._STable.HOW;
 
       var method = how.find_method(null, null, how, this, name);
 
+      if (method === Null) {
+        methodNotFoundError(arguments[0], arguments[2], name);
+      }
+
       var args = [];
       for (var i = 0; i < arguments.length; i++) {
         args.push(arguments[i]);
@@ -61,13 +103,13 @@ function basicConstructor(STable) {
   };
 
 
-  objConstructor.prototype = Object.create(new Proxy({}, handler));
-  objConstructor.prototype._STable = STable;
+  ObjConstructor.prototype = Object.create(new Proxy({}, handler));
+  ObjConstructor.prototype._STable = STable;
 
-  objConstructor.prototype._SC = undefined;
-  objConstructor.prototype._WHERE = undefined;
+  ObjConstructor.prototype._SC = undefined;
+  ObjConstructor.prototype._WHERE = undefined;
 
-  return objConstructor;
+  return ObjConstructor;
 }
 
 function slotToAttr(slot) {
@@ -83,7 +125,7 @@ REPR.prototype.createObjConstructor = basicConstructor;
 
 class P6opaque {
   allocate(STable) {
-    var obj = new STable.objConstructor();
+    var obj = new STable.ObjConstructor();
     obj.$$setDefaults();
     return obj;
   }
@@ -109,10 +151,7 @@ class P6opaque {
     this.unboxNumSlot = cursor.varint();
     this.unboxStrSlot = cursor.varint();
 
-
-
     var hasUnboxSlots = cursor.varint();
-
     if (hasUnboxSlots != 0) {
       this.unboxSlots = [];
       for (var i = 0; i < numAttributes; i++) {
@@ -186,7 +225,7 @@ class P6opaque {
   getHint(classHandle, attrName) {
     var hint = this.hintfor(classHandle, attrName);
     if (hint == -1) {
-      throw new NQPException("Can't find: " + attrName);
+      throw new NQPException(`Can't find: ${attrName}`);
     } else {
       return hint;
     }
@@ -203,10 +242,9 @@ class P6opaque {
     for (var i = 0; i < numAttrs; i++) {
       if (st.REPR.flattenedSTables[i] == null) {
         cursor.varint(0);
-      }
-      else {
+      } else {
         cursor.varint(1);
-        cursor.STableRef(st.REPR.flattenedSTables[i]);
+        cursor.stableRef(st.REPR.flattenedSTables[i]);
       }
     }
 
@@ -262,26 +300,14 @@ class P6opaque {
   }
 
   deserializeFinish(obj, data) {
-    var attrs = [];
-
     for (var i = 0; i < this.flattenedSTables.length; i++) {
-      if (this.flattenedSTables[i]) {
-        attrs.push(this.flattenedSTables[i].REPR.deserializeInline(data));
-      } else {
-        attrs.push(data.variant());
-      }
-    }
-
-    for (var mapping of this.nameToIndexMapping) {
-      for (var slot of mapping.slots) {
-        obj[slotToAttr(slot)] = attrs[slot];
-      }
+      let attr = this.flattenedSTables[i] ? this.flattenedSTables[i].REPR.deserializeInline(data) : data.variantWithUndefined();
+      obj[slotToAttr(i)] = attr;
     }
   }
 
   serialize(cursor, obj) {
     var flattened = obj._STable.REPR.flattenedSTables;
-    var nqp = require('nqp-runtime');
     if (!flattened) {
       throw 'Representation must be composed before it can be serialized';
     }
@@ -290,7 +316,7 @@ class P6opaque {
       var value = obj[slotToAttr(i)];
 
       if (flattened[i] == null) {
-        cursor.ref(value === undefined ? Null : value);
+        cursor.ref(value);
       } else {
         flattened[i].REPR.serializeInline(cursor, value);
       }
@@ -315,12 +341,10 @@ class P6opaque {
       }
     }
 
-    Object.setPrototypeOf(obj, newType._STable.objConstructor.prototype);
+    Object.setPrototypeOf(obj, newType._STable.ObjConstructor.prototype);
   }
 
   compose(STable, reprInfoHash) {
-    // TODO
-
     /* Get attribute part of the protocol from the hash. */
     var reprInfo = reprInfoHash.content.get('attribute').array;
 
@@ -414,6 +438,7 @@ class P6opaque {
 
     STable.compileAccessor('$$bindattr$' + slot, 'function(value) {\n' +
         'this.' + attr + ' = value;\n' +
+        'if (this._SC !== undefined) this.$$scwb();\n' +
         'return value;\n' +
         '}\n');
 
@@ -438,50 +463,52 @@ class P6opaque {
           '}\n'
       );
     }
-
   }
 
-  generateDefaults(STable) {
-    var code = '';
+  generateDefaultsAndClone(STable) {
+    let defaults = '';
+    let clone = '';
 
-    for (var i = 0; i < this.nameToIndexMapping.length; i++) {
-      for (var j = 0; j < this.nameToIndexMapping[i].slots.length; j++) {
-        let slot = this.nameToIndexMapping[i].slots[j];
-        let defaultValue = this.flattenedSTables[slot] ?
-            this.flattenedSTables[slot].REPR.flattenedDefault :
-            'undefined';
-        code += 'this.' + slotToAttr(slot) + ' = ' + defaultValue + ';\n';
-      }
+    for (let slot = 0; slot < this.flattenedSTables.length; slot++) {
+      let attr = slotToAttr(slot);
+      let defaultValue = this.flattenedSTables[slot] ?
+          this.flattenedSTables[slot].REPR.flattenedDefault :
+          'undefined';
+      defaults += 'this.' + attr + ' = ' + defaultValue + ';\n';
+      clone += 'cloned.' + attr + ' = this.' + attr + ';';
     }
 
-    STable.compileAccessor('$$setDefaults', 'function() {\n' + code + '}');
+    STable.compileAccessor('$$setDefaults', 'function() {\n' + defaults + '}');
+    STable.compileAccessor('$$clone', 'function() {var cloned = new this._STable.ObjConstructor();' + clone + 'return cloned}');
     STable.evalGatheredCode();
   }
 
   generateUniversalAccessors(STable) {
     this.generateUniversalAccessor(STable, '$$getattr', function(slot) {
       return 'return this.$$getattr$' + slot + '()';
-    }, '');
+    }, '', false);
 
     this.generateUniversalAccessor(STable, '$$bindattr', function(slot) {
       return 'return this.$$bindattr$' + slot + '(value)';
-    }, ', value');
+    }, ', value', false);
 
     var suffixes = ['_s', '_i', '_n'];
     for (let suffix of suffixes) {
       /* TODO only check attributes of proper type */
       this.generateUniversalAccessor(STable, '$$getattr' + suffix, function(slot) {
         return 'return this.' + slotToAttr(slot);
-      }, '');
+      }, '', false);
 
       this.generateUniversalAccessor(STable, '$$bindattr' + suffix, function(slot) {
         return 'return (this.' + slotToAttr(slot) + ' = value)';
-      }, ', value');
+      }, ', value', true);
     }
   }
 
-  generateUniversalAccessor(STable, name, action, extraSig) {
-    var code = 'function(classHandle, attrName' + extraSig + ') {\n' + 'switch (classHandle) {\n';
+  generateUniversalAccessor(STable, name, action, extraSig, scwb) {
+    var code = 'function(classHandle, attrName' + extraSig + ') {\n' +
+        (scwb ? 'if (this._SC !== undefined) this.$$scwb();\n' : '') +
+        'switch (classHandle) {\n';
     var classKeyIndex = 0;
     var setup = '';
     if (this.nameToIndexMapping) {
@@ -515,7 +542,7 @@ class P6opaque {
 
     this.generateUniversalAccessors(STable);
 
-    this.generateDefaults(STable);
+    this.generateDefaultsAndClone(STable);
 
     STable.evalGatheredCode();
   }
@@ -528,7 +555,6 @@ class P6opaque {
         return (this[attr] == undefined) ? 0 : 1;
       }
     });
-
   }
 
 };
@@ -552,7 +578,7 @@ class KnowHOWREPR {
   }
 
   allocate(STable) {
-    var obj = new STable.objConstructor();
+    var obj = new STable.ObjConstructor();
     obj.__methods = new Hash();
     obj.__attributes = [];
     obj.__name = '<anon>';
@@ -564,7 +590,6 @@ KnowHOWREPR.prototype.typeObjectFor = basicTypeObjectFor;
 KnowHOWREPR.prototype.createObjConstructor = basicConstructor;
 
 
-
 reprs.KnowHOWREPR = KnowHOWREPR;
 
 class KnowHOWAttribute {
@@ -591,26 +616,47 @@ class Uninstantiable extends REPR {
 reprs.Uninstantiable = Uninstantiable;
 
 class P6int extends REPR {
+  constructor() {
+    super();
+    this.bits = 32;
+    this.isUnsigned = 0;
+  }
+
   setupSTable(STable) {
     STable.addInternalMethods(class {
       $$setInt(value) {
         this.value = value;
       }
-      $$getInt(value) {
+
+      $$getInt() {
+        return this.value;
+      }
+
+      $$decont_i(value) {
         return this.value;
       }
     });
   }
 
   compose(STable, reprInfoHash) {
-    var integer = reprInfoHash.content.get('integer');
+    let integer = reprInfoHash.content.get('integer');
     if (integer) {
-      var bits = integer.content.get('bits');
-      if (bits instanceof NQPInt) {
+      let bits = integer.content.get('bits');
+      if (bits === undefined) {
+      } else if (bits instanceof NQPInt) {
         this.bits = bits.value;
       } else {
         throw 'bits to P6int.compose must be a native int';
       }
+
+      let unsigned = integer.content.get('unsigned');
+      if (unsigned) {
+        if (unsigned instanceof NQPInt) {
+          this.isUnsigned = unsigned.value;
+        } else {
+          throw 'unsigned to P6int.compose must be a native int';
+        }
+      }
     }
   }
 
@@ -642,6 +688,10 @@ class P6int extends REPR {
       $$getInt() {
         return this[name];
       }
+
+      $$decont_i(ctx) {
+        return this[name];
+      }
     });
   }
 
@@ -652,6 +702,17 @@ class P6int extends REPR {
       return new NQPInt(this[attr]);
     });
   }
+
+
+  serializeReprData(st, cursor) {
+    cursor.varint(this.bits);
+    cursor.varint(this.isUnsigned);
+  }
+
+  deserializeReprData(cursor) {
+    this.bits = cursor.varint();
+    this.isUnsigned = cursor.varint();
+  }
 };
 
 P6int.prototype.flattenedDefault = '0';
@@ -673,6 +734,10 @@ class P6num extends REPR {
       $$getNum() {
         return this.value;
       }
+
+      $$decont_n(value) {
+        return this.value;
+      }
     });
   }
 
@@ -701,6 +766,10 @@ class P6num extends REPR {
       $$getNum() {
         return this[name];
       }
+
+      $$decont_n(ctx) {
+        return this[name];
+      }
     });
   }
 
@@ -731,6 +800,10 @@ class P6str extends REPR {
       $$getStr() {
         return this.value;
       }
+
+      $$decont_s(value) {
+        return this.value;
+      }
     });
   }
 
@@ -759,8 +832,11 @@ class P6str extends REPR {
       $$getStr() {
         return this[name];
       }
-    });
 
+      $$decont_s(ctx) {
+        return this[name];
+      }
+    });
   }
 
   generateFlattenedAccessors(ownerSTable, attrContentSTable, slot) {
@@ -773,14 +849,119 @@ class P6str extends REPR {
 
 P6str.prototype.boxedPrimitive = 3;
 P6str.prototype.flattenSTable = true;
-P6str.prototype.flattenedDefault = 'null_s';
+P6str.prototype.flattenedDefault = 'nullStr';
 
 
 reprs.P6str = P6str;
 
 class NFA extends REPR {
   deserializeFinish(obj, data) {
-    // STUB
+    /* Read fates. */
+
+    obj.fates = data.variant();
+
+    /* Read number of states. */
+
+    let numStates = data.varint();
+
+    /* Read state graph. */
+
+    obj.states = [];
+
+    let edgeCount = [];
+
+    for (let i = 0; i < numStates; i++) {
+      edgeCount[i] = data.varint();
+    }
+
+    for (let i = 0; i < numStates; i++) {
+      obj.states[i] = [];
+      for (let j = 0; j < edgeCount[i]; j++) {
+        var edge = {act: data.varint(), to: data.varint()};
+        switch (edge.act & 0xff) {
+          case EDGE_EPSILON:
+            break;
+          case EDGE_FATE:
+          case EDGE_CODEPOINT:
+          case EDGE_CODEPOINT_LL:
+          case EDGE_CODEPOINT_NEG:
+          case EDGE_CHARCLASS:
+          case EDGE_CHARCLASS_NEG:
+            edge.argI = data.varint();
+            break;
+          case EDGE_CHARLIST:
+          case EDGE_CHARLIST_NEG:
+            edge.argS = data.str();
+            break;
+
+          case EDGE_CODEPOINT_I:
+          case EDGE_CODEPOINT_I_LL:
+          case EDGE_CODEPOINT_I_NEG:
+          case EDGE_CHARRANGE:
+          case EDGE_CHARRANGE_NEG:
+            edge.argLc = data.varint();
+            edge.argUc = data.varint();
+            break;
+          default:
+            throw 'NFA deserialization: unknown codepoint type: ' + edge.act;
+        }
+        obj.states[i].push(edge);
+      }
+    }
+  }
+
+  serialize(cursor, obj) {
+    /* Write fates. */
+
+    cursor.ref(obj.fates);
+
+    /* Write number of states. */
+
+    cursor.varint(obj.states.length);
+
+    /* Write state edge list counts. */
+
+    for (let i = 0; i < obj.states.length; i++) {
+      cursor.varint(obj.states[i].length);
+    }
+
+    /* Write state graph. */
+
+    for (let i = 0; i < obj.states.length; i++) {
+      for (let j = 0; j < obj.states[i].length; j++) {
+        let edge = obj.states[i][j];
+
+        cursor.varint(edge.act);
+        cursor.varint(edge.to);
+
+        switch (edge.act & 0xff) {
+          case EDGE_EPSILON:
+            break;
+          case EDGE_FATE:
+          case EDGE_CODEPOINT:
+          case EDGE_CODEPOINT_LL:
+          case EDGE_CODEPOINT_NEG:
+          case EDGE_CHARCLASS:
+          case EDGE_CHARCLASS_NEG:
+            cursor.varint(edge.argI);
+            break;
+          case EDGE_CHARLIST:
+          case EDGE_CHARLIST_NEG:
+            cursor.str(edge.argS);
+            break;
+          case EDGE_CODEPOINT_I:
+          case EDGE_CODEPOINT_I_LL:
+          case EDGE_CODEPOINT_I_NEG:
+          case EDGE_CHARRANGE:
+          case EDGE_CHARRANGE_NEG:
+            cursor.varint(edge.argLc);
+            cursor.varint(edge.argUc);
+            break;
+          default:
+            throw 'NFA serialization - unknown codepoint type: ' + edge.act;
+        }
+      }
+    }
   }
 };
 
@@ -790,13 +971,13 @@ reprs.NFA = NFA;
 class VMArray extends REPR {
 
   allocate(STable) {
-    var obj = new STable.objConstructor();
+    var obj = new STable.ObjConstructor();
     obj.array = [];
     return obj;
   }
 
   allocateFromArray(STable, array) {
-    var obj = new STable.objConstructor();
+    var obj = new STable.ObjConstructor();
     obj.array = array;
     return obj;
   }
@@ -804,6 +985,7 @@ class VMArray extends REPR {
   setupSTable(STable) {
     STable.addInternalMethods(class {
       $$push(value) {
+        if (this._SC !== undefined) this.$$scwb();
         this.array.push(value);
         return value;
       }
@@ -823,7 +1005,7 @@ class VMArray extends REPR {
 
       $$atpos_s(index) {
         var value = this.array[index < 0 ? this.array.length + index : index];
-        if (value === undefined) return null_s;
+        if (value === undefined) return nullStr;
         return value;
       }
 
@@ -840,11 +1022,16 @@ class VMArray extends REPR {
       }
 
       $$bindpos(index, value) {
+        if (this._SC !== undefined) this.$$scwb();
         return this.array[index < 0 ? this.array.length + index : index] = value;
       }
 
       $$join(delim) {
-        return this.array.join(delim);
+        var stringified = [];
+        for (let i = 0; i < this.array.length; i++) {
+          stringified.push(typeof this.array[i] == 'string' ? this.array[i] : this.array[i].$$getStr());
+        }
+        return stringified.join(delim);
       }
 
 
@@ -906,18 +1093,27 @@ class VMArray extends REPR {
         return this.array.length;
       }
 
-      $$splice(source, offset, length) {
+      $$splice(source, offset, count) {
         // TODO think about the case when the source is not VMArray
-        var args = [offset, length];
-        for (var i = 0; i < source.array.length; i++) {
-          args.push(source.array[i]);
+        if (count < source.array.length) {
+           this.array.length = this.array.length + source.array.length - count;
         }
-        this.array.splice.apply(this.array, args);
+
+        this.array.copyWithin(offset + source.array.length, offset + count);
+
+        for (let i = 0; i < source.array.length; i++) {
+          this.array[offset + i] = source.array[i];
+        }
+
+        if (count > source.array.length) {
+           this.array.length = this.array.length + source.array.length - count;
+        }
+
         return this;
       }
 
       $$clone() {
-        var cloned = new STable.objConstructor();
+        var cloned = new STable.ObjConstructor();
         cloned.array = this.array.slice();
         return cloned;
       }
@@ -981,16 +1177,6 @@ class VMArray extends REPR {
     cursor.ref(this.type);
   }
 
-  deserializeArray(obj, data) {
-    if (this.type !== Null) {
-      console.log('NYI: VMArrays of a type different then null');
-    }
-    var size = data.varint();
-    for (var i = 0; i < size; i++) {
-      obj.array[i] = data.variant();
-    }
-  }
-
   compose(STable, reprInfoHash) {
     if (reprInfoHash.content.get('array')) {
       this.type = reprInfoHash.content.get('array').content.get('type') || Null;
@@ -1002,6 +1188,87 @@ class VMArray extends REPR {
 
 reprs.VMArray = VMArray;
 
+let HashIter = require('./hash-iter.js');
+
+class VMHash extends REPR {
+  allocate(STable) {
+    var obj = new STable.ObjConstructor();
+    obj.content = new Map();
+    return obj;
+  }
+
+  setupSTable(STable) {
+    STable.addInternalMethods(class {
+      $$clone() {
+        var cloned = new STable.ObjConstructor();
+        cloned.content = new Map();
+        this.content.forEach(function(value, key, map) {
+          cloned.content.set(key, value);
+        });
+        return cloned;
+      }
+
+      $$bindkey(key, value) {
+        this.content.set(key, value);
+        if (this._SC !== undefined) this.$$scwb();
+        return value;
+      }
+
+      $$atkey(key) {
+        return this.content.has(key) ? this.content.get(key) : Null;
+      }
+
+      $$existskey(key) {
+        return this.content.has(key);
+      }
+
+      $$deletekey(key) {
+        if (this._SC !== undefined) this.$$scwb();
+        this.content.delete(key);
+        return this;
+      }
+
+      $$elems() {
+        return this.content.size;
+      }
+
+      $$numify() {
+        return this.$$elems();
+      }
+
+      $$iterator() {
+        return new HashIter(this);
+      }
+
+      $$toObject() {
+        var ret = {};
+        this.content.forEach(function(value, key, map) {
+          ret[key] = value;
+        });
+        return ret;
+      }
+    });
+  }
+
+  deserializeFinish(obj, data) {
+    obj.content = new Map();
+    let elems = data.varint();
+    for (var i = 0; i < elems; i++) {
+      let str = data.str();
+      obj.content.set(str, data.variant());
+    }
+  }
+
+  serialize(cursor, obj) {
+    cursor.varint(obj.$$elems());
+    obj.content.forEach(function(value, key, map) {
+      cursor.str(key);
+      cursor.ref(value);
+    }, this);
+  }
+};
+
+reprs.VMHash = VMHash;
 
 class VMIter {
   deserializeFinish(obj, data) {
@@ -1043,6 +1310,10 @@ class P6bigint extends REPR {
       $$getBignum() {
         return this.value;
       }
+
+      $$decont_i(ctx) {
+        return this.value.toNumber() | 0;
+      }
     });
   }
 
@@ -1105,7 +1376,11 @@ class P6bigint extends REPR {
       }
 
       $$getInt() {
-        return this[name].toNumber();
+        return this[name].toNumber() | 0;
+      }
+
+      $$decont_i(ctx) {
+        return this[name].toNumber() | 0;
       }
 
       $$getBignum() {
@@ -1155,12 +1430,155 @@ reprs.Semaphore = Semaphore;
 class ConcBlockingQueue extends REPR {};
 reprs.ConcBlockingQueue = ConcBlockingQueue;
 
-class Decoder extends REPR {};
+const emptyBuffer = Buffer.allocUnsafe(0);
+
+const StringDecoder = require('string_decoder').StringDecoder;
+
+const defaultSeps = ['\r\n', '\n'];
+
+class Decoder extends REPR {
+  setupSTable(STable) {
+    STable.addInternalMethods(class {
+      $$check() {
+        if (this.$$encoding === undefined) {
+          throw new NQPException('Decoder not yet configured');
+        }
+      }
+
+      $$translate(str) {
+        return this.$$shouldTranslate ? str.replace(/\r\n/g, '\n') : str;
+      }
+
+      $$decoderconfigure(ctx, encoding, config) {
+        if (this.$$encoding !== undefined) {
+          throw new NQPException('Decoder already configured');
+        }
+        this.$$encoding = core.renameEncoding(encoding);
+        if (!core.isKnownEncoding(this.$$encoding)) {
+          throw new NQPException(`Unknown string encoding: '${this.$$encoding}'`);
+        }
+        let translate = config.content.get('translate_newlines');
+        this.$$shouldTranslate = translate && nqp.toInt(translate, ctx) !== 0;
+
+        this.$$seps = defaultSeps;
+        this.$$buffer = emptyBuffer;
+        return this;
+      }
+
+      $$decodersetlineseps(ctx, seps) {
+        this.$$check();
+        this.$$seps = seps.array.map(sep => nqp.toStr(sep, ctx));
+      }
+
+      $$decoderaddbytes(bytes) {
+        this.$$check();
+        let buf = core.toRawBuffer(bytes);
+        this.$$buffer = Buffer.concat([this.$$buffer, buf]);
+      }
+
+      /* TODO NFG, codepoints that don't fit into 2 bytes */
+      $$decodertakechars(count) {
+        this.$$check();
+        let decoder = new StringDecoder(this.$$encoding);
+        let everything = decoder.write(this.$$buffer);
+        let chars = everything.substr(0, count);
+        if (chars.length < count) {
+          return nullStr;
+        } else {
+          this.$$buffer = this.$$buffer.slice(Buffer.byteLength(chars, this.$$encoding));
+          return this.$$translate(chars);
+        }
+      }
+
+      $$decodertakeavailablechars() {
+        this.$$check();
+        let decoder = new StringDecoder(this.$$encoding);
+        let available = decoder.write(this.$$buffer);
+        this.$$buffer = this.$$buffer.slice(0, Buffer.byteLength(available, available));
+        return this.$$translate(available);
+      }
+
+      $$decodertakeallchars() {
+        this.$$check();
+        let ret = this.$$buffer.toString(this.$$encoding);
+        this.$$buffer = emptyBuffer;
+        return this.$$translate(ret);
+      }
+
+      /* TODO: NFG */
+
+      $$decodertakeline(chomp, incompleteOk) {
+        this.$$check();
+        let decoder = new StringDecoder(this.$$encoding);
+        let string = decoder.write(this.$$buffer);
+
+        let newline = -1;
+        let sep;
+        for (let i = 0; i < this.$$seps.length; i++) {
+          let offset = string.indexOf(this.$$seps[i]);
+          if (offset != -1 && (newline == -1 || offset < newline)) {
+            newline = offset;
+            sep = this.$$seps[i];
+          }
+        }
+
+        if (newline != -1) {
+          let upToNewline = string.slice(0, newline);
+
+          this.$$buffer = this.$$buffer.slice(Buffer.byteLength(upToNewline + sep, this.$$encoding));
+
+          return this.$$translate(chomp ? upToNewline : upToNewline + sep);
+        } else {
+          if (incompleteOk) {
+            let ret = this.$$buffer.toString(this.$$encoding);
+            this.$$buffer = emptyBuffer;
+            return this.$$translate(ret);
+          } else {
+            return nullStr;
+          }
+        }
+      }
+
+      $$decoderempty() {
+        this.$$check();
+        return this.$$buffer.length === 0 ? 1 : 0;
+      }
+
+      $$decoderbytesavailable() {
+        this.$$check();
+        return this.$$buffer.length;
+      }
+
+      $$decodertakebytes(bufType, bytes) {
+        this.$$check();
+        if (bytes > this.$$buffer.length) {
+          return Null;
+        }
+
+        let buf = bufType._STable.REPR.allocate(bufType._STable);
+
+        let elementSize = core.byteSize(buf);
+        let isUnsigned = buf._STable.REPR.type._STable.REPR.isUnsigned;
+
+
+        let offset = 0;
+        for (var i = 0; i < bytes / elementSize; i++) {
+          buf.array[i] = isUnsigned ? this.$$buffer.readUIntLE(offset, elementSize) : this.$$buffer.readIntLE(offset, elementSize);
+          offset += elementSize;
+        }
+
+        this.$$buffer = this.$$buffer.slice(bytes);
+
+        return buf;
+      }
+    });
+  }
+};
 reprs.Decoder = Decoder;
 
 class MultiDimArray extends REPR {
   allocate(STable) {
-    var obj = new STable.objConstructor();
+    var obj = new STable.ObjConstructor();
     obj.dimensions = undefined;
     return obj;
   }
@@ -1184,7 +1602,6 @@ class MultiDimArray extends REPR {
       if (dimensions === 0) {
         throw new NQPException('MultiDimArray REPR must be composed with at least 1 dimension');
       }
-
     } else {
       throw 'dimensions to MultiDimArray.compose must be a native int';
     }
@@ -1194,7 +1611,6 @@ class MultiDimArray extends REPR {
   }
 
 
-
   setupSTable(STable) {
     STable.addInternalMethods(class {
       $$numdimensions(value) {
@@ -1205,7 +1621,7 @@ class MultiDimArray extends REPR {
       }
 
       $$clone() {
-        var clone = new this._STable.objConstructor();
+        var clone = new this._STable.ObjConstructor();
         clone.storage = this.storage.slice();
         clone.dimensions = this.dimensions;
         return clone;
@@ -1244,7 +1660,7 @@ class MultiDimArray extends REPR {
         throw new NQPException('Cannot push a fixed dimension array');
       }
 
-      $$splice(value) {
+      $$splice(source, offset, length) {
         throw new NQPException('Cannot splice a fixed dimension array');
       }
 
@@ -1405,10 +1821,9 @@ reprs.MultiDimArray = MultiDimArray;
 
 class VMException extends REPR {
   setupSTable(STable) {
-
     STable.addInternalMethods(class {
       $$getStr() {
-        return this.message;
+        return this.$$message;
       }
     });
   }
@@ -1442,6 +1857,7 @@ var ID = 0;
 for (var name in reprs) {
   module.exports[name] = reprs[name];
   reprs[name].prototype.ID = ID;
+  reprs[name].prototype.name = name;
   reprById[ID] = reprs[name];
   if (reprs[name].prototype.flattenedDefault) {
     reprs[name].prototype.flattenedDefaultObj = eval(reprs[name].prototype.flattenedDefault);
