@@ -3,10 +3,12 @@ var Hash = require('./hash.js');
 var CodeRef = require('./code-ref.js');
 var NQPInt = require('./nqp-int.js');
 var Int64 = require('node-int64');
-var null_s = require('./null_s.js');
+var nullStr = require('./null_s.js');
 var Null = require('./null.js');
-var MultiCache = require('./multicache.js').MultiCache;
-var FileHandle = require('./io.js').FileHandle;
+
+const NQPException = require('./nqp-exception.js');
+
+var StaticCtx = require('./static-ctx.js');
 
 var repossession = require('./repossession.js');
 
@@ -18,8 +20,6 @@ var op = {};
 exports.op = op;
 
 const CURRENT_VERSION = 20;
-const OBJECTS_TABLE_ENTRY_SC_MASK = 0x7FF;
-const OBJECTS_TABLE_ENTRY_SC_IDX_MASK = 0x000FFFFF;
 const OBJECTS_TABLE_ENTRY_SC_IDX_MAX = 0x000FFFFF;
 const OBJECTS_TABLE_ENTRY_SC_MAX = 0x7FE;
 const OBJECTS_TABLE_ENTRY_SC_SHIFT = 20;
@@ -46,7 +46,6 @@ const REFVAR_VM_HASH_STR_VAR = 10;
 const REFVAR_STATIC_CODEREF = 11;
 const REFVAR_CLONED_CODEREF = 12;
 
-const STABLE_BOOLIFICATION_SPEC_MODE_MASK = 0x0F;
 const STABLE_HAS_CONTAINER_SPEC = 0x10;
 const STABLE_HAS_INVOCATION_SPEC = 0x20;
 const STABLE_HAS_HLL_OWNER = 0x40;
@@ -67,75 +66,6 @@ class BinaryWriteCursor {
     this.currentMetadata = [];
   }
 
-  // TODO disable when not needed, Object.assign is not fully portable
-
-  start(tag) {
-    var meta = {tag: tag, start: this.offset};
-
-    this.metadata.push(meta);
-    this.currentMetadata.push(meta);
-  }
-
-  end(tag, extra) {
-    this.currentMetadata[this.currentMetadata.length - 1].end = this.offset;
-    this.currentMetadata.pop();
-  }
-
-  tagged() {
-    var out = '';
-    var events = [];
-    for (var i = 0; i < this.metadata.length; i++) {
-      events.push({offset: this.metadata[i].start, start: this.metadata[i], i: i});
-      events.push({offset: this.metadata[i].end, end: this.metadata[i], i: i});
-    }
-
-    events.sort(function(a, b) {
-      if (a.offset < b.offset) return -1;
-      if (a.offset > b.offset) return 1;
-
-      if (a.end && b.start) return -1;
-      if (a.start && b.end) return 1;
-
-      if (a.start && b.start) {
-        if (a.i < b.i) return -1;
-        if (a.i > b.i) return 1;
-      } else {
-        if (a.i < b.i) return 1;
-        if (a.i > b.i) return -1;
-      }
-
-      return 0;
-    });
-
-    var offset = 0;
-    for (var i = 0; i < events.length; i++) {
-      var e = events[i];
-
-      var bytes = [];
-      while (offset < e.offset) {
-        bytes.push(this.buffer.readUInt8(offset));
-        offset++;
-      }
-      out += bytes.join(' ');
-
-      if (e.start) {
-        out += '<' + e.start.tag + '>';
-      } else if (e.end) {
-        out += '</' + e.end.tag + '>';
-      }
-
-    }
-
-    var bytes = [];
-    while (offset < this.offset) {
-      bytes.push(this.buffer.readUInt8(offset));
-      offset++;
-    }
-    out += bytes.join(' ');
-
-    return out;
-  }
-
   growToHold(space) {
     if (this.offset + space > this.buffer.length) {
       var old = this.buffer;
@@ -153,20 +83,22 @@ class BinaryWriteCursor {
     var heapLoc = this.writer.stringIndex(str);
 
     if (!(heapLoc >= 0 && heapLoc <= STRING_HEAP_LOC_MAX)) {
-      throw 'Serialization error: string offset ' + heapLoc + "can't be serialized";
+      throw `Serialization error: string offset ${heapLoc} can't be serialized`;
     }
 
     if (heapLoc <= STRING_HEAP_LOC_PACKED_MAX) {
-      this.U16(heapLoc);
+      this.uint16(heapLoc);
     } else {
-      this.U16((heapLoc >> STRING_HEAP_LOC_PACKED_SHIFT) |
+      this.uint16((heapLoc >> STRING_HEAP_LOC_PACKED_SHIFT) |
           STRING_HEAP_LOC_PACKED_OVERFLOW);
-      this.U16(heapLoc & STRING_HEAP_LOC_PACKED_LOW_MASK);
+      this.uint16(heapLoc & STRING_HEAP_LOC_PACKED_LOW_MASK);
     }
   }
 
 
-  /** Write a C String */
+  /** Write a C String
+   * @param {string} string
+   */
   cstr(string) {
     if (string === undefined) {
       this.varint(0);
@@ -183,13 +115,12 @@ class BinaryWriteCursor {
   }
 
   str32(str) {
-    this.I32(this.writer.stringIndex(str));
+    this.int32(this.writer.stringIndex(str));
   }
 
-  /* Writing function for variable sized integers. Writes out a 64 bit value
-     using between 1 and 9 bytes. */
+  /* Writing function for variable sized integers. Writes out a 32 bit value
+     using between 1 and 5 bytes. */
   varint(value) {
-    this.start('varint');
     var storageNeeded;
 
     if (value >= -1 && value <= 126) {
@@ -197,32 +128,19 @@ class BinaryWriteCursor {
     } else {
       var absVal = value < 0 ? -value - 1 : value;
 
-      if (absVal <= 0x7FF)
+      if (absVal <= 0x7FF) {
         storageNeeded = 2;
-      else if (absVal <= 0x000000000007FFFF)
+      } else if (absVal <= 0x000000000007FFFF) {
         storageNeeded = 3;
-      else if (absVal <= 0x0000000007FFFFFF)
+      } else if (absVal <= 0x0000000007FFFFFF) {
         storageNeeded = 4;
-      else if (absVal <= 0x00000007FFFFFFFF)
+      } else if (absVal <= 0x00000007FFFFFFFF) {
         storageNeeded = 5;
-      else console.log('TODO serializing bigger integers');
-
-      /* TODO bigger numbers */
-      /*else if (absVal <= 0x000007FFFFFFFFFFLL)
-              storageNeeded = 6;
-          else if (absVal <= 0x0007FFFFFFFFFFFFLL)
-              storageNeeded = 7;
-          else if (absVal <= 0x07FFFFFFFFFFFFFFLL)
-              storageNeeded = 8;
-          else
-              storageNeeded = 9;*/
+      }
     }
 
     if (storageNeeded == 1) {
-      this.U8(0x80 | (value + 129));
-    } else if (storageNeeded == 9) {
-      this.I8(0x00);
-      this.I64(value);
+      this.uint8(0x80 | (value + 129));
     } else {
       var rest = storageNeeded - 1;
       var nybble = rest == 4 ? 0 : value >> 8 * rest;
@@ -232,7 +150,7 @@ class BinaryWriteCursor {
 
       console.assert((nybble >> 3) == 0 || (nybble >> 3) == ~0);
 
-      this.I8((rest << 4) | (nybble & 0xF));
+      this.int8((rest << 4) | (nybble & 0xF));
 
       var tmp = new Int64(value).toBuffer();
       this.growToHold(rest);
@@ -242,50 +160,35 @@ class BinaryWriteCursor {
 
       this.offset += rest;
     }
-
-    this.end();
   }
 
 
-  I8(value) {
+  int8(value) {
     this.growToHold(1);
     this.buffer.writeInt8(value, this.offset);
     this.offset += 1;
   }
 
-  U8(value) {
+  uint8(value) {
     this.growToHold(1);
     this.buffer.writeUInt8(value, this.offset);
     this.offset += 1;
   }
 
-  U16(value) {
+  uint16(value) {
     this.growToHold(2);
     this.buffer.writeUInt16LE(value, this.offset);
     this.offset += 2;
   }
 
 
-  I32(value) {
-    this.start('I32');
-
+  int32(value) {
     this.growToHold(4);
     this.buffer.writeInt32LE(value, this.offset);
     this.offset += 4;
-
-    this.end();
   }
 
-
-  /* TODO - numbers bigger than 32bit */
-  I64(value) {
-    this.growToHold(8);
-    this.buffer.writeInt32LE(value, this.offset);
-    this.buffer.writeInt32LE(0, this.offset + 4);
-    this.offset += 8;
-  }
-
-  I16(value) {
+  int16(value) {
     this.growToHold(2);
     this.buffer.writeInt16LE(value, this.offset);
     this.offset += 2;
@@ -298,8 +201,6 @@ class BinaryWriteCursor {
   }
 
 
-
-
   /* Writes the ID, index pair that identifies an entry in a Serialization
      context. */
   idIdx(scId, idx) {
@@ -318,11 +219,7 @@ class BinaryWriteCursor {
   objRef(ref) {
     var writerSc = this.writer.sc;
     if (!ref._STable) {
-      console.log(ref);
-      console.trace("can't serialize this for sure");
-      console.log(typeof ref);
-      console.log(ref.codeRef());
-      process.exit();
+      throw new NQPException('trying to serialize an object without an STable');
     }
     if (!ref._SC) {
       /* This object doesn't belong to an SC yet, so it must be serialized
@@ -332,17 +229,11 @@ class BinaryWriteCursor {
       this.writer.sc.rootObjects.push(ref);
     }
 
-    var sc = ref._SC;
-    if (!sc) {
-      console.log('!sc', !ref._SC, ref._SC);
-      console.trace('!sc');
-      //process.exit();
-    }
     /* Write SC index, then object index. */
-    this.idIdx(this.writer.getSCId(sc), sc.rootObjects.indexOf(ref));
+    this.idIdx(this.writer.getSCId(ref._SC), ref._SC.rootObjects.indexOf(ref));
   }
 
-  STableRef(STable) {
+  stableRef(STable) {
     var ref = this.writer.getSTableRefInfo(STable);
     this.idIdx(ref[0], ref[1]);
   }
@@ -353,67 +244,42 @@ class BinaryWriteCursor {
 
     if (ref == Null) {
       discrim = REFVAR_VM_NULL;
-    }
-    else if (ref instanceof FileHandle) {
+    } else if (ref === undefined) {
+      discrim = REFVAR_NULL;
+    } else if (ref.$$serializeAsNull) {
       discrim = REFVAR_VM_NULL;
-    }
-  //  else if (ref.st.REPR instanceof CallCapture) {
-  //      /* This is a hack for Rakudo's sake; it keeps a CallCapture around in
-  //       * the lexpad, for no really good reason. */
-  //      discrim = REFVAR_VM_NULL;
-  //  }
-
-    else if (ref instanceof MultiCache) {
-      discrim = REFVAR_VM_NULL;
-    }
-
-    else if (ref instanceof NQPInt) {
+    } else if (ref instanceof NQPInt) {
       discrim = REFVAR_VM_INT;
-    }
-    else if (typeof ref == 'number') {
+    } else if (typeof ref == 'number') {
       discrim = REFVAR_VM_NUM;
-    }
-    else if (typeof ref == 'string') {
+    } else if (typeof ref == 'string') {
       discrim = REFVAR_VM_STR;
-    }
-    else if (ref._STable === BOOT.Array._STable) {
+    } else if (ref._STable === BOOT.Array._STable) {
       discrim = REFVAR_VM_ARR_VAR;
-    }
-  //  else if (ref.st.WHAT == tc.gc.BOOTIntArray) {
-  //      discrim = REFVAR_VM_ARR_INT;
-  //  }
-  //  else if (ref.st.WHAT == tc.gc.BOOTStrArray) {
-  //      discrim = REFVAR_VM_ARR_STR;
-  //  }
-    else if (ref instanceof Hash) {
+    } else if (ref instanceof Hash) {
       discrim = REFVAR_VM_HASH_STR_VAR;
-    }
-    else if (ref instanceof CodeRef || typeof ref == 'function') {
-      //      console.log("serializing code ref");
+    } else if (ref instanceof CodeRef || typeof ref == 'function') {
       discrim = REFVAR_VM_NULL;
       if (ref._SC && ref.isStatic) {
         /* Static code reference. */
         discrim = REFVAR_STATIC_CODEREF;
-      }
-      else if (ref._SC) {
+      } else if (ref._SC) {
         /* Closure, but already seen and serialization already handled. */
         discrim = REFVAR_CLONED_CODEREF;
-      }
-      else {
+      } else {
         /* Closure but didn't see it yet. Take care of it serialization, which
              * gets it marked with this SC. Then it's just a normal code ref that
              * needs serializing. */
         this.writer.serializeClosure(ref);
         discrim = REFVAR_CLONED_CODEREF;
       }
-    }
-    else {
+    } else {
       /* Just a normal object, with no special serialization needs. */
       discrim = REFVAR_OBJECT;
     }
 
 
-    this.I8(discrim);
+    this.int8(discrim);
 
     /* Now take appropriate action. */
     switch (discrim) {
@@ -433,17 +299,12 @@ class BinaryWriteCursor {
       case REFVAR_VM_STR:
         this.str(ref);
         break;
-      //      case REFVAR_VM_ARR_INT:
-      //      case REFVAR_VM_ARR_STR:
-      //          ref.st.REPR.serialize(tc, this, ref);
+      case REFVAR_VM_ARR_INT:
+      case REFVAR_VM_ARR_STR:
       case REFVAR_VM_ARR_VAR:
-        this.varint(ref.array.length);
-        for (var i = 0; i < ref.array.length; i++) {
-          this.ref(ref.array[i] || Null);
-        }
+        ref._STable.REPR.serialize(this, ref);
         break;
       case REFVAR_VM_HASH_STR_VAR:
-        var count = 0;
         this.varint(ref.$$elems());
         ref.content.forEach(function(value, key, map) {
           this.str(key);
@@ -455,12 +316,12 @@ class BinaryWriteCursor {
         var scId = this.writer.getSCId(ref._SC);
         var idx = ref._SC.rootCodes.indexOf(ref);
         if (idx == -1) {
-          throw "can't write code ref";
+          throw `can't write code ref`;
         }
         this.idIdx(scId, idx);
         break;
       default:
-        throw 'Serialization Error: Unimplemented object type: ' + discrim;
+        throw `Serialization Error: Unimplemented object type: ${discrim}`;
     }
   }
 };
@@ -473,7 +334,7 @@ class SerializationWriter {
     this.contexts = [];
     this.sc = sc;
     this.sh = sh;
-    this.sh.push(null_s);
+    this.sh.push(nullStr);
     this.numClosures = 0;
 
     this.stables = new BinaryWriteCursor(this);
@@ -490,9 +351,7 @@ class SerializationWriter {
   serializeObject(obj) {
     /* Get index of SC that holds the STable and its index. */
     if (!obj._STable) {
-      console.log('obj:', obj.constructor.name, obj);
-      console.trace("can't serialize");
-      //process.exit();
+      throw new NQPException(`Can't serialize an object without an STable`);
     }
     var ref = this.getSTableRefInfo(obj._STable);
     var sc = ref[0];
@@ -504,21 +363,20 @@ class SerializationWriter {
       packed |= (sc << OBJECTS_TABLE_ENTRY_SC_SHIFT) | scIdx;
     } else {
       packed |= OBJECTS_TABLE_ENTRY_SC_OVERFLOW << OBJECTS_TABLE_ENTRY_SC_SHIFT;
-      this.objectsData.I32(sc);
-      this.objectsData.I32(scIdx);
+      this.objectsData.int32(sc);
+      this.objectsData.int32(scIdx);
     }
 
     /* Make objects table entry. */
 
-    this.objects.I32(packed);
-    this.objects.I32(this.objectsData.offset);
-
+    this.objects.int32(packed);
+    this.objects.int32(this.objectsData.offset);
 
 
     /* Delegate to its serialization REPR function. */
     if (!obj.typeObject_) {
       if (!obj._STable.REPR.serialize) {
-        console.trace("don't know how to serialize");
+        console.trace(`don't know how to serialize ${obj._STable.REPR.name}`);
       } else {
         obj._STable.REPR.serialize(this.objectsData, obj);
       }
@@ -529,11 +387,9 @@ class SerializationWriter {
    * its representation data also. */
 
   serializeSTable(st) {
-    this.stablesData.start('stable');
-
     /* Make STables table entry. */
     this.stables.str32(st.REPR.constructor.name);
-    this.stables.I32(this.stablesData.offset);
+    this.stables.int32(this.stablesData.offset);
 
     /* Write HOW, WHAT and WHO. */
 
@@ -549,13 +405,10 @@ class SerializationWriter {
 
     /* Method cache*/
     if (st.methodCache) {
-      var hash = new Hash();
-      for (var key in st.methodCache) {
-        hash.content.set(key, st.methodCache[key]);
-      }
+      let hash = new Hash();
+      hash.content = st.methodCache;
       this.stablesData.ref(hash);
-    }
-    else {
+    } else {
       this.stablesData.ref(Null);
     }
 
@@ -577,7 +430,7 @@ class SerializationWriter {
        convey, and hence show buggy behaviour. And if we're bumping the
        serialisation version, then we can increase the storage size.  */
 
-    this.stablesData.U8(st.modeFlags);
+    this.stablesData.uint8(st.modeFlags);
 
     /* Boolification spec. */
     /* As this only needs 4 bits, also use the same byte to store various
@@ -588,7 +441,7 @@ class SerializationWriter {
     if (st.boolificationSpec) {
       var mode = st.boolificationSpec.mode;
       if (mode >= 0xF) {
-        throw 'Serialization error: boolification spec mode ' + mode + " out of range and can't be serialized";
+        throw `Serialization error: boolification spec mode ${mode} out of range and can't be serialized`;
       }
       flags = mode;
     } else {
@@ -611,7 +464,7 @@ class SerializationWriter {
       flags |= STABLE_HAS_HLL_ROLE;
     }
 
-    this.stablesData.U8(flags);
+    this.stablesData.uint8(flags);
 
     /* Boolification spec. */
 
@@ -670,14 +523,12 @@ class SerializationWriter {
     this.stablesData.cstr(st.debugName);
 
     /* Location of REPR data. */
-    this.stables.I32(this.stablesData.offset);
+    this.stables.int32(this.stablesData.offset);
 
     /* If the REPR has a function to serialize representation data, call it. */
     if (st.REPR.serializeReprData) {
       st.REPR.serializeReprData(st, this.stablesData);
     }
-
-    this.stablesData.end();
   }
 
 
@@ -693,20 +544,19 @@ class SerializationWriter {
 
 
     /* Make contexts table entry. */
-    this.contextsHeaders.I32(staticSCId);
-    this.contextsHeaders.I32(staticIdx);
-    this.contextsHeaders.I32(this.contextsData.offset);
+    this.contextsHeaders.int32(staticSCId);
+    this.contextsHeaders.int32(staticIdx);
+    this.contextsHeaders.int32(this.contextsData.offset);
 
     /* See if there's any relevant outer context, and if so set it up to
      * be serialized. */
     if (ctx.$$outer != null) {
-      this.contextsHeaders.I32(this.getSerializedContextIdx(ctx.$$outer));
+      this.contextsHeaders.int32(this.getSerializedContextIdx(ctx.$$outer));
     } else {
-      this.contextsHeaders.I32(0);
+      this.contextsHeaders.int32(0);
     }
 
 
-
     var lexicalsTypeInfo = staticCodeRef.lexicalsTypeInfo;
 
     var lexicals = 0;
@@ -734,17 +584,22 @@ class SerializationWriter {
 
   getSerializedContextIdx(ctx) {
     if (!ctx._SC) {
+      /* TODO - think if it's truly correct */
+
+      if (ctx instanceof StaticCtx) {
+        /* A StaticCtx is used when a closure is invoke when it's outer doesn't exist yet */
+        return 0;
+      }
+
       /* Make sure we should chase a level down. */
       if (this.closureToStaticCodeRef(ctx.codeRef(), false) == null) {
         return 0;
-      }
-      else {
+      } else {
         this.contexts.push(ctx);
         ctx._SC = this.sc;
         return this.contexts.length;
       }
-    }
-    else {
+    } else {
       if (ctx._SC != this.sc) {
         throw 'Serialization Error: reference to context outside of SC';
       }
@@ -757,10 +612,12 @@ class SerializationWriter {
   }
 
   getSerializedOuterContextIdx(closure) {
-    if (closure.isCompilerStub)
+    if (closure.isCompilerStub) {
       return 0;
-    if (closure.outerCtx == null)
+    }
+    if (closure.outerCtx == null) {
       return 0;
+    }
     return this.getSerializedContextIdx(closure.outerCtx);
   }
 
@@ -798,15 +655,15 @@ class SerializationWriter {
      * needed). */
     var contextIdx = this.getSerializedOuterContextIdx(closure);
 
-    this.closures.I32(staticSCId);
-    this.closures.I32(staticIdx);
-    this.closures.I32(contextIdx);
+    this.closures.int32(staticSCId);
+    this.closures.int32(staticIdx);
+    this.closures.int32(contextIdx);
 
 
     /* Check if it has a static code object. */
     if (closure.codeObj) {
       var codeObj = closure.codeObj;
-      this.closures.I32(1);
+      this.closures.int32(1);
 
       if (!codeObj._SC) {
       }
@@ -815,13 +672,12 @@ class SerializationWriter {
         this.writer.sc.rootObjects.push(ref);
       }
 
-      this.closures.I32(this.getSCId(codeObj._SC));
-      this.closures.I32(codeObj._SC.rootObjects.indexOf(codeObj));
-    }
-    else {
-      this.closures.I32(0);
-      this.closures.I32(0);
-      this.closures.I32(0);
+      this.closures.int32(this.getSCId(codeObj._SC));
+      this.closures.int32(codeObj._SC.rootObjects.indexOf(codeObj));
+    } else {
+      this.closures.int32(0);
+      this.closures.int32(0);
+      this.closures.int32(0);
     }
 
     /* Increment count of closures in the table. */
@@ -894,20 +750,20 @@ class SerializationWriter {
           origSC.rootSTables.indexOf(this.sc.rootSTables[objIdx]) :
           origSC.rootObjects.indexOf(this.sc.rootObjects[objIdx]);
 
-      if (origIdx < 0)
+      if (origIdx < 0) {
         throw 'Could not find object when writing repossessions; ' +
-            (isST != 0 ? 'STable' : 'REPR = ' + this.sc.rootObjects[objIdx]._STable.REPR.name);
+          (isST != 0 ? 'STable' : 'REPR = ' + this.sc.rootObjects[objIdx]._STable.REPR.name);
+      }
 
       /* Write table row. */
-      this.repossessionsData.I32(isST);
-      this.repossessionsData.I32(objIdx);
-      this.repossessionsData.I32(origSCIdx);
-      this.repossessionsData.I32(origIdx);
+      this.repossessionsData.int32(isST);
+      this.repossessionsData.int32(objIdx);
+      this.repossessionsData.int32(origSCIdx);
+      this.repossessionsData.int32(origIdx);
     }
   }
 
 
-
   stringIndex(str) {
     /* The first entry in the heap represents the null string */
     var idx = this.sh.indexOf(str);
@@ -919,7 +775,7 @@ class SerializationWriter {
   }
 
 
-  header_I32(value) {
+  headerInt32(value) {
     this.buffer.writeUInt32LE(value, this.headerOffset);
     this.headerOffset += 4;
   }
@@ -933,17 +789,19 @@ class SerializationWriter {
 
   getSCId(sc) {
     if (!sc) {
-      console.trace('undefined sc');
-      process.exit();
+      throw new NQPException('getting an id of an undefined SC');
     }
+
     /* Easy if it's in the current SC. */
-    if (sc == this.sc)
+    if (sc == this.sc) {
       return 0;
+    }
 
     /* If not, try to find it in our dependencies list. */
     var found = this.dependentSCs.indexOf(sc);
-    if (found >= 0)
+    if (found >= 0) {
       return found + 1;
+    }
 
     /* Otherwise, need to add it to our dependencies list. */
     this.dependentSCs.push(sc);
@@ -986,65 +844,66 @@ class SerializationWriter {
     this.buffer = new Buffer(outputSize);
 
     /* Write version into header. */
-    this.header_I32(CURRENT_VERSION);
+    this.headerInt32(CURRENT_VERSION);
 
     /* Put dependencies table in place and set location/rows in header. */
-    this.header_I32(this.offset);
-    this.header_I32(this.dependentSCs.length);
+    this.headerInt32(this.offset);
+    this.headerInt32(this.dependentSCs.length);
 
 
     this.writeChunk(this.deps);
 
     /* Put STables table in place, and set location/rows in header. */
-    this.header_I32(this.offset);
-    this.header_I32(this.sc.rootSTables.length);
+    this.headerInt32(this.offset);
+    this.headerInt32(this.sc.rootSTables.length);
 
     this.writeChunk(this.stables);
 
     /* Put STables data in place. */
-    this.header_I32(this.offset);
+    this.headerInt32(this.offset);
 
     this.writeChunk(this.stablesData);
 
     /* Put objects table in place, and set location/rows in header. */
 
-    this.header_I32(this.offset);
-    this.header_I32(this.sc.rootObjects.length);
+    this.headerInt32(this.offset);
+    this.headerInt32(this.sc.rootObjects.length);
 
     this.writeChunk(this.objects);
 
     /* Put objects data in place. */
-    this.header_I32(this.offset);
+    this.headerInt32(this.offset);
 
     this.writeChunk(this.objectsData);
 
     /* Put closures table in place, and set location/rows in header. */
-    this.header_I32(this.offset);
-    this.header_I32(this.numClosures);
+    this.headerInt32(this.offset);
+    this.headerInt32(this.numClosures);
 
     this.writeChunk(this.closures);
 
     /* Put contexts table in place, and set location/rows in header. */
-    this.header_I32(this.offset);
-    this.header_I32(this.contexts.length);
+    this.headerInt32(this.offset);
+    this.headerInt32(this.contexts.length);
 
     this.writeChunk(this.contextsHeaders);
 
     /* Put contexts data in place. */
-    this.header_I32(this.offset);
+    this.headerInt32(this.offset);
 
     this.writeChunk(this.contextsData);
 
     /* Put repossessions table in place, and set location/rows in header. */
-    this.header_I32(this.offset);
-    this.header_I32(this.sc.repScs.length);
+    this.headerInt32(this.offset);
+    this.headerInt32(this.sc.repScs.length);
 
     this.writeChunk(this.repossessionsData);
 
 
-    //  /* Sanity check. */
-    if (this.offset != outputSize)
-      throw 'Serialization sanity check failed: offset != outputSize (' + offset + ' != ' + outputSize + ')';
+    /* Sanity check. */
+    if (this.offset != outputSize) {
+      throw `Serialization sanity check failed: offset != outputSize (${offset} != ${outputSize})`;
+    }
 
     return this.buffer.toString('base64');
   }
@@ -1075,8 +934,9 @@ op.getobjsc = function(obj) {
 
 op.scgetobjidx = function(sc, obj) {
   var idx = sc.rootObjects.indexOf(obj);
-  if (idx < 0)
+  if (idx < 0) {
     throw 'Object does not exist in this SC';
+  }
   return idx;
 };
 
